@@ -54,9 +54,6 @@ def create_logger(
         environment=env,
     )
 
-    # Instruments, capture additionnal info
-    logfire.instrument_pymongo()
-
     # Create a logger instance with the service name as a tag
     logger = logfire.with_settings(tags=[logger_name, env])
 
